/***************************************************************************
                       cdemu_kernel.h  -  description
                             -------------------
    begin                : 2002-12-29
    copyright            : 2003 by Robert Penz <robert.penz@outertech.com>
                           2003 by Mike Frysinger <vapier@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef __CDEMU_KERNEL_H__
#define __CDEMU_KERNEL_H__

#define MAJOR_NR 0
#define	DEVICE_NR(device)			MINOR(device)	/* has no partition bits */
#define	DEVICE_NAME					"cdemu"			/* name for messaging */
#define	DEVICE_NO_RANDOM							/* no entropy to contribute */
#define	DEVICE_REQUEST				cdemu_request
#define	DEVICE_OFF(d)								/* do-nothing */

#include <linux/module.h>			/* try_module_get,module_put */
#include <linux/init.h>				/* init/exit funcs */
#include <linux/genhd.h>			/* struct gendisk */
#include <linux/blkdev.h>			/* block device operations */
#include <linux/genhd.h>			/* the generic disk interface */
#include <linux/cdrom.h>			/* cdrom declarations */
#include <linux/file.h>				/* fput */
#include <linux/highmem.h>			/* kmap functions */
#include <asm/current.h>			/* access to the 'current' struct */
#include <linux/moduleparam.h>		/* MODULE_AUTHOR,module_parm */
#include <linux/buffer_head.h>		/* buffer operations */
#include <linux/bio.h>
#include <scsi/sg.h>

#ifdef CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
#endif

#include "cdemu_proc.h"
#include "cdemu.h"
#define	CDEMU_MODULE_VERSION		CDEMU_VERSION

struct cdemu_device {
	int							number;
	dev_t						device;
	tTrack						tracks[CDEMU_MAX_TRACKS];
	int							num_tracks;		/* number of tracks on the cd */
	struct file					*backing_file;
	int							loaded;
	int							uid;			/* user who mounted cd */
	int							media_changed;
	int							locked;
	int							pause;
	char						comment[CDEMU_COMMENT_SIZE];
	int							size;
	spinlock_t					lock;
	struct cdrom_device_info	info;
	struct request_queue		*queue;
	struct gendisk				*gd;
	struct block_device			*bdev;
};

#define CDEMU_DEBUG
#ifdef CDEMU_DEBUG
#define	dprintk(msg, args...) \
	do { \
		printk(KERN_DEBUG "cdemu:%i: " msg, __LINE__, ## args); \
	} while(0)
#else
#define	dprintk(msg, args...)
#endif
#define printk_cdemu(lvl, msg, args...) \
	do { \
		printk(lvl "cdemu:%i: " msg, __LINE__, ## args); \
	} while(0)

#define	AUDIO						0
#define	MODE1						1
#define	MODE2						2
#define AUDIO_2448                  10
#define MODE1_2448                  20
#define	MODE1_2352					30
#define MODE1_2048                  40
#define	MODE2_2352					50
#define	MODE2_2336					60

#define	CDEMU_LOADCD				0x5400			/* load the virtual cd */
#define	CDEMU_UNLOADCD				0x5401			/* unload the virtual cd*/

extern int cdemu_major, cdemu_count;
extern struct cdemu_device *cdemu_dev;

#define	UP_MODULE_USE()		try_module_get(THIS_MODULE)
#define	DOWN_MODULE_USE()	module_put(THIS_MODULE)

#endif /* __CDEMU_KERNEL_H__ */
